@@ -42,7 +42,7 @@
  *----------------------------------------------------------------------*/
 
 #ifndef lint
-static const char rcsid[] = "$Id: command.c,v 1.20 1998/09/24 18:22:36 mason Exp $";
+static const char rcsid[] = "$Id: command.c,v 1.5 2000/05/28 07:46:35 sashav Exp $";
 #endif
 
 /*{{{ includes: */
@@ -2201,15 +2201,8 @@ process_x_event(XEvent * ev)
 #ifdef OFF_FOCUS_FADING
 	    if( rs_fade != NULL )
 	    {
-	      extern int focus_changed ;
-	        focus_changed = 1 ;
     		PixColors = &(PixColorsFocused[0]);
-		if (!(Options & Opt_transparent))
-		{
-		    XSetWindowBackground(Xdisplay, TermWin.vt, PixColors[Color_bg]);
-		    scr_clear();
-		}
-		scr_touch();
+		on_colors_changed(Color_bg);
 	    }
 #endif
 #ifndef NO_XLOCALE
@@ -2225,15 +2218,8 @@ process_x_event(XEvent * ev)
 #ifdef OFF_FOCUS_FADING
 	    if( rs_fade != NULL )
 	    {
-	      extern int focus_changed ;
-	        focus_changed = 1 ;  
 		PixColors = &(PixColorsUnFocused[0]); 
-		if (!(Options & Opt_transparent))
-		{
-		    XSetWindowBackground(Xdisplay, TermWin.vt, PixColors[Color_bg]);
-		    scr_clear();
-		}
-		scr_touch();
+		on_colors_changed(Color_bg);
 	    }
 #endif
 #ifndef NO_XLOCALE
@@ -2260,9 +2246,28 @@ process_x_event(XEvent * ev)
 	selection_send(&(ev->xselectionrequest));
 	break;
 	
-#ifdef TRANSPARENT
      case PropertyNotify:
-	
+#ifdef _MYSTYLE_
+	if (ev->xproperty.atom == _AS_STYLE ) 
+	{
+	  const char *mystyle_name ;
+	  MyStyle *mystyle ;
+
+	    if( rs_mystyle ) 
+		mystyle_name = rs_mystyle ;
+	    else 
+		mystyle_name = GetDefaultMyStyle() ;
+
+	    mystyle_get_property(Xdisplay, Xroot, _AS_STYLE, XA_INTEGER );
+	    
+	    set_mystyle( mystyle_find( mystyle_name ), True );
+	    break;	    
+	}
+	else if( TermWin.background.trgType == BGT_MyStyle && 
+		 TermWin.background.mystyle->texture_type < TEXTURE_TRANSPARENT)
+	    break ;
+#endif
+#ifdef TRANSPARENT
         refresh_transparent_scrollbar();
         if( Options & Opt_transparent )
 	{
@@ -2309,8 +2314,8 @@ process_x_event(XEvent * ev)
 		    if( TermWin.bMapped ) LastRPixmap = currentRPixmap ;
 		}
     	} /* case */
-	break;
 #endif /* TRANSPARENT */
+	break;
      case UnmapNotify:
         TermWin.bMapped = 0 ; 
 /*	fprintf(stderr, "\n aterm is UnMapped(event)");*/
@@ -2374,8 +2379,6 @@ process_x_event(XEvent * ev)
 
 		if (Options & Opt_transparent)     
 		    XSetWindowBackgroundPixmap(Xdisplay, parent, ParentRelative);
-
-
 	    }
 	    if( XGetWindowAttributes( Xdisplay, ParentWin[0], &attr ))
 		TermWin.bMapped = (attr.map_state == IsViewable)?1:0 ;
