@@ -32,11 +32,12 @@
  *----------------------------------------------------------------------*/
 
 #ifndef lint
-static const char rcsid[] = "$Id: main.c,v 1.18 1998/09/24 19:22:44 mason Exp $";
+static const char rcsid[] = "$Id: main.c,v 1.12 2000/06/09 05:36:27 sashav Exp $";
 #endif
 
 #define INTERN			/* assign all global vars to me */
 #include "rxvt.h"		/* NECESSARY */
+#include "X11/Xatom.h"
 
 Window ParentWin[PARENTS_NUM] = PARENTS_INIT;
 int    ParentWinNum = 0;
@@ -123,18 +124,6 @@ static const char *def_fontName[] =
  * while window resizing                                 */
 static int      font_change_count = 0;
 
-/* some global stuff that needs to be defined for libasimage */
-#ifdef USE_LIBASIMAGE
-Display* dpy ;
-char* MyName = "aterm";
-int DeadPipe( int nonsense ){return 1;}
-typedef struct ScreenInfo
-{
-    unsigned long screen; /* your default screen */
-}ScreenInfo;
-ScreenInfo Scr;    
-#endif
-
 
 /*----------------------------------------------------------------------*/
 /* ARGSUSED */
@@ -228,6 +217,34 @@ set_colorfgbg(void)
 # define set_colorfgbg() ((void)0)
 #endif				/* NO_BRIGHTCOLOR */
 /*}}} */
+
+/* PROTO */
+void
+set_terminal_size( unsigned int new_ncol, unsigned int new_nrow )
+{
+    if( new_nrow <= 0) new_nrow = 24 ;
+    if( new_ncol <= 0) new_ncol = 80 ;
+    TermWin.ncol = new_ncol;
+    TermWin.nrow = new_nrow;
+    TermWin.bcol = new_ncol;
+    MAX_IT(TermWin.bcol, TermWin.min_bcol); 
+}
+
+
+/*{{{ set_cursor_color() - Updates color of the TerWin_cursor */
+/* PROTO */
+void 
+set_cursor_color()
+{
+    XColor          fg, bg;
+
+    fg.pixel = PixColorsFocused[Color_pointer];
+    XQueryColor(Xdisplay, Xcmap, &fg);
+    bg.pixel = PixColorsFocused[Color_bg];
+    XQueryColor(Xdisplay, Xcmap, &bg);
+    XRecolorCursor(Xdisplay, TermWin_cursor, &fg, &bg);
+}
+/*}}} */
 #if defined(BACKGROUND_IMAGE)
 int SetBackgroundPixmap(char* PixmapSpec)
 {
@@ -240,17 +257,25 @@ int SetBackgroundPixmap(char* PixmapSpec)
         bChanged = parse_pixmap_geom(geometry+1);
     }
     LoadBGPixmap(PixmapSpec);
+#ifdef _MYSTYLE_    
+    if( TermWin.background.trgType == BGT_MyStyle ) 
+	TermWin.background.trgType = BGT_Tile ;
+#endif
+
     if( geometry ) *geometry = ';' ;
     return bChanged ;
 }
 #endif
 
-#if defined(BACKGROUND_IMAGE) || defined(TRANSPARENT)
+#if defined(BACKGROUND_IMAGE) || defined(TRANSPARENT) || defined(_MYSTYLE_)
 
 void SetBackgroundType(const char* type)
 {
 
-    TermWin.background.trgType = BGT_Tile ;
+#ifdef _MYSTYLE_
+    if( TermWin.background.trgType != BGT_MyStyle || type != NULL )
+#endif
+	TermWin.background.trgType = BGT_Tile ;
     
 #ifdef TRANSPARENT
     if( (Options & Opt_transparent) && !type )
@@ -284,9 +309,11 @@ void SetBackgroundType(const char* type)
 
 }
 
-void InitBackground(const char* type, const char* shading)
+void InitBackground()
 {
     TermWin.background.srcPixmap = None ;
+    TermWin.background.mystyle = NULL ;
+    TermWin.background.user_flags = 0 ;
     TermWin.background.bMySource = 0 ;                
     TermWin.background.trgPixmap = None ;    
     TermWin.background.trgPixmapSet = 0 ;
@@ -310,12 +337,6 @@ void InitBackground(const char* type, const char* shading)
     /* some defaults here */
     INIT_SHADING(TermWin.background.Shading)
     
-    if( shading )   
-        TermWin.background.Shading.shading = atoi(shading);
-	
-    SetBackgroundType( type );
-    
-    
 }
 #endif
 
@@ -338,6 +359,28 @@ int ParseGCType( const char* type, int def_type )
     return def_type ;
 }
 
+#ifdef OFF_FOCUS_FADING
+unsigned long fade_color(unsigned long pixel)
+{
+    if( rs_fade )
+    {/* make unfocused colors here */
+      XColor          faded_xcol;
+      int fade = 0;
+	
+        fade = atoi( rs_fade );
+        faded_xcol.pixel = pixel ;
+	XQueryColor( Xdisplay, Xcmap, &faded_xcol );
+        faded_xcol.red   = (faded_xcol.red/100)*fade ;
+        faded_xcol.green = (faded_xcol.green/100)*fade ;	      
+        faded_xcol.blue  = (faded_xcol.blue/100)*fade ;	    
+	    
+        if( XAllocColor(Xdisplay, Xcmap, &faded_xcol) )
+    	    return faded_xcol.pixel ;
+    }
+    return pixel ;
+}
+#endif
+
 /*{{{ Create_Windows() - Open and map the window */
 /* PROTO */
 void
@@ -348,20 +391,38 @@ Create_Windows(int argc, char *argv[])
     XWMHints        wmHint;
     int             i, x, y, flags;
     unsigned int    width, height;
-#ifdef OFF_FOCUS_FADING    
-    int fade = 0;
-#endif
 #ifdef PREFER_24BIT
     XSetWindowAttributes attributes;
+    XWindowAttributes gattr; 
+        Xcmap = DefaultColormap(Xdisplay, Xscreen);
+        Xvisual = DefaultVisual(Xdisplay, Xscreen);
+
+    if (Options & Opt_transparent) {
+        XGetWindowAttributes(Xdisplay, RootWindow(Xdisplay, Xscreen), &gattr);
+        Xdepth = gattr.depth;
+    } else {
+        Xdepth = DefaultDepth(Xdisplay, Xscreen);
+ /*
+  * If depth is not 24, look for a 24bit visual.
+  */
+        if (Xdepth != 24) {
+             XVisualInfo        vinfo;
+ 
+             if (XMatchVisualInfo(Xdisplay, Xscreen, 24, TrueColor, &vinfo)) {
+                Xdepth = 24;
+                Xvisual = vinfo.visual;
+                Xcmap = XCreateColormap(Xdisplay,
+                                        RootWindow(Xdisplay, Xscreen), Xvisual,
+                                        AllocNone);
+             }
+	}    
+    }
 #endif
 
 /*
  * grab colors before netscape does
  */
     PixColors = &(PixColorsFocused[0]);
-#ifdef OFF_FOCUS_FADING    
-    if( rs_fade != NULL )   fade = atoi( rs_fade );
-#endif
 
     for (i = 0;
 	 i < (Xdepth <= 2 ? 2 : NRS_COLORS);
@@ -415,19 +476,7 @@ Create_Windows(int argc, char *argv[])
 	}
 	PixColors[i] = xcol.pixel;
 #ifdef OFF_FOCUS_FADING
-	if( rs_fade )
-	{/* make unfocused colors here */
-	  XColor          faded_xcol;
-
-	    faded_xcol.red   = (xcol.red/100)*fade ;
-	    faded_xcol.green = (xcol.green/100)*fade ;	      
-	    faded_xcol.blue  = (xcol.blue/100)*fade ;	    
-	    
-	    if( XAllocColor(Xdisplay, Xcmap, &faded_xcol) )
-		PixColorsUnFocused[i] = faded_xcol.pixel ;
-	    else 
-		PixColorsUnFocused[i] = xcol.pixel ; 
-	}
+	PixColorsUnFocused[i] = fade_color(xcol.pixel);
 #endif
 #if defined(TRANSPARENT) || defined(BACKGROUND_IMAGE)	
 	if( i == Color_tint )
@@ -520,8 +569,8 @@ Create_Windows(int argc, char *argv[])
 	szHint.height = height;
 	szHint.flags |= USSize;
     }
-    TermWin.ncol = szHint.width;
-    TermWin.nrow = szHint.height;
+    
+    set_terminal_size( szHint.width, szHint.height );
 
     change_font(1, NULL);
 
@@ -559,7 +608,7 @@ Create_Windows(int argc, char *argv[])
     TermWin.parent = XCreateWindow(Xdisplay, Xroot,
 				   szHint.x, szHint.y,
 				   szHint.width, szHint.height,
-				   BORDERWIDTH,
+                                   TermWin.borderWidth,
 				   Xdepth, InputOutput,
 				   Xvisual,
 				   CWBackPixel | CWBorderPixel | CWColormap,
@@ -568,7 +617,7 @@ Create_Windows(int argc, char *argv[])
     TermWin.parent = XCreateSimpleWindow(Xdisplay, Xroot,
 					 szHint.x, szHint.y,
 					 szHint.width, szHint.height,
-					 BORDERWIDTH,
+					 TermWin.borderWidth,
 					 PixColors[Color_bg],
 					 PixColors[Color_fg]);
 #endif
@@ -625,10 +674,16 @@ Create_Windows(int argc, char *argv[])
 #ifdef TRANSPARENT
     if (Options & Opt_transparent) {
 	XSetWindowBackgroundPixmap(Xdisplay, TermWin.parent, ParentRelative);
-	XSetWindowBackgroundPixmap(Xdisplay, TermWin.vt, ParentRelative);
+	if( TermWin.background.trgType != BGT_None ) 
+	    SetSrcPixmap(GetRootPixmap(None));
+	else XSetWindowBackgroundPixmap(Xdisplay, TermWin.vt, ParentRelative);
     }
+#endif
     /* added by Sasha Vasko to enabling Tracking of the Background changes */
+#if defined(TRANSPARENT)||defined(_MYSTYLE_)
+#if !defined(_MYSTYLE_)
     if ((Options & Opt_transparent)||(Options&Opt_transparent_sb)) 
+#endif
 	XSelectInput( Xdisplay, Xroot, PropertyChangeMask );
     
 #endif
@@ -798,8 +853,8 @@ resize_window1(unsigned int width, unsigned int height)
 	    selection_clear();
 	    curr_screen = scr_change_screen(PRIMARY);
 	}
-	TermWin.ncol = new_ncol;
-	TermWin.nrow = new_nrow;
+	
+	set_terminal_size( new_ncol, new_nrow);
 
 	resize_subwindows(width, height);
 	scr_reset();
@@ -849,7 +904,7 @@ resize_window(XEvent* ev)
 
     resize_window1((ev->xconfigure).width, (ev->xconfigure).height);
 
-#if defined(TRANSPARENT) || defined(BACKGROUND_IMAGE)
+#if defined(TRANSPARENT) || defined(BACKGROUND_IMAGE) || defined(_MYSTYLE_)
     XGetGeometry(Xdisplay, TermWin.vt,
 		 &root, &x, &y, &vt_width, &vt_height, &border, &depth);
 
@@ -878,10 +933,31 @@ resize_window(XEvent* ev)
 
 	    if( GetMyPosition(&abs_x, &abs_y))
 	    {
-	    /*fprintf( stderr, "\naterm:resize_windows():old = %dx%d+%d+%d, new = %dx%d+%d+%d",
+/*	    fprintf( stderr, "\naterm:resize_windows():old = %dx%d+%d+%d, new = %dx%d+%d+%d",
 		     old_width, old_height, old_abs_x, old_abs_y,
 		     vt_width, vt_height, abs_x, abs_y   );  
-	    */	if( TermWin.background.trgType != BGT_None )
+*/
+#ifdef _MYSTYLE_
+	        if( TermWin.background.trgType == BGT_MyStyle )
+		{ /* we want to avoid unneeded redraws if we 
+		     only have gradients */
+		    if( TermWin.background.mystyle->texture_type < TEXTURE_TRANSPARENT )
+		    {
+    	    		if( (vt_width != old_width || vt_height!= old_height )
+			    && TermWin.background.mystyle->texture_type > TEXTURE_SOLID
+			    && TermWin.background.mystyle->texture_type < TEXTURE_PIXMAP )
+			{
+			    RenderPixmap(1); 
+			    old_width = vt_width ;
+			    old_height = vt_height ;
+			}
+    		        scr_clear();
+			scr_touch();	
+			return ; 
+		    }
+		}
+#endif
+		if( TermWin.background.trgType != BGT_None )
 		{
 		    ValidateSrcPixmap( 1 );
 	    	    if( abs_x != old_abs_x || abs_y != old_abs_y ||
@@ -978,12 +1054,12 @@ set_window_color(int idx, const char *color)
 	if (i >= 8 && i <= 15) {	/* bright colors */
 	    i -= 8;
 # ifndef NO_BRIGHTCOLOR
-	    PixColors[idx] = PixColors[minBrightCOLOR + i];
+	    PixColorsFocused[idx] = PixColorsFocused[minBrightCOLOR + i];
 	    goto Done;
 # endif
 	}
 	if (i >= 0 && i <= 7) {	/* normal colors */
-	    PixColors[idx] = PixColors[minCOLOR + i];
+	    PixColorsFocused[idx] = PixColorsFocused[minCOLOR + i];
 	    goto Done;
 	}
     }
@@ -1000,37 +1076,24 @@ set_window_color(int idx, const char *color)
  */
 # if 0
     for (i = Color_Black; i <= Color_White; i++)
-	if (PixColors[idx] == PixColors[i])
+	if (PixColorsFocused[idx] == PixColorsFocused[i])
 	    break;
     if (i > Color_White) {
     /* fprintf (stderr, "XFreeColors: PixColors [%d] = %lu\n", idx, PixColors [idx]); */
-	XFreeColors(Xdisplay, Xcmap, (PixColors + idx), 1,
+	XFreeColors(Xdisplay, Xcmap, (PixColorsFocused + idx), 1,
 		    DisplayPlanes(Xdisplay, Xscreen));
     }
 # endif
 
-    PixColors[idx] = xcol.pixel;
+    PixColorsFocused[idx] = xcol.pixel;
 
 /* XSetWindowAttributes attr; */
 /* Cursor cursor; */
   Done:
-    if (idx == Color_bg && !(Options & Opt_transparent))
-	XSetWindowBackground(Xdisplay, TermWin.vt, PixColors[Color_bg]);
-
-/* handle Color_BD, scrollbar background, etc. */
-
-    set_colorfgbg();
-    {
-	XColor          fg, bg;
-
-	fg.pixel = PixColors[Color_pointer];
-	XQueryColor(Xdisplay, Xcmap, &fg);
-	bg.pixel = PixColors[Color_bg];
-	XQueryColor(Xdisplay, Xcmap, &bg);
-	XRecolorCursor(Xdisplay, TermWin_cursor, &fg, &bg);
-    }
-/* the only reasonable way to enforce a clean update */
-    scr_poweron();
+#ifdef OFF_FOCUS_FADING
+    PixColorsUnFocused[idx] = fade_color(PixColorsFocused[idx]);
+#endif
+    on_colors_changed(idx);
 }
 #else
 # define set_window_color(idx,color)	((void)0)
@@ -1101,6 +1164,24 @@ xterm_seq(int op, const char *str)
 }
 /*}}} */
 
+
+unsigned int get_proportion_font_width( XFontStruct *font )
+{
+  int i, cw ;
+  unsigned int fw = 0;
+    if( font != NULL )
+    {
+	for (i = font->max_char_or_byte2 - 
+	         font->min_char_or_byte2 ;
+	     i >= 0; i--) 
+	{
+	    cw = font->per_char[i].width;
+	    if( cw > 0 )
+    	        MAX_IT(fw, cw);
+	}
+    }
+    return fw ;
+}
 /*{{{ change_font() - Switch to a new font */
 /*
  * init = 1   - initialize
@@ -1263,11 +1344,7 @@ change_font(int init, const char *fontname)
 	    TermWin.fprop = 1;	/* Proportional font */
 
 	if (TermWin.fprop == 1)	/* also, if == 0, per_char[] might be NULL */
-	    for (i = TermWin.font->min_char_or_byte2;
-		 i <= TermWin.font->max_char_or_byte2; i++) {
-		cw = TermWin.font->per_char[i].width;
-		MAX_IT(fw, cw);
-	    }
+	    fw = get_proportion_font_width( TermWin.font );
     /* not the first time thru and sizes haven't changed */
 	if (fw == TermWin.fwidth && fh == TermWin.fheight)
 	    return;		/* TODO: not return; check MULTI if needed */
@@ -1290,11 +1367,8 @@ change_font(int init, const char *fontname)
 	    if (fw != boldFont->max_bounds.width)
 		fw = -1;
 	} else
-	    for (i = TermWin.font->min_char_or_byte2;
-		 i <= TermWin.font->max_char_or_byte2; i++) {
-		cw = boldFont->per_char[i].width;
-		MAX_IT(fw, cw);
-	    }
+	    fw = get_proportion_font_width( boldFont );
+
 	if (fw == TermWin.fwidth && fh == TermWin.fheight)
 	    TermWin.boldFont = boldFont;
     }
@@ -1335,6 +1409,135 @@ change_font(int init, const char *fontname)
 }
 /*}}} */
 
+#ifdef _MYSTYLE_
+char *DefaultMyStyle = NULL ;
+
+/* PROTO */
+const char* 
+GetDefaultMyStyle()
+{
+    if( DefaultMyStyle == NULL )
+    {
+	DefaultMyStyle = malloc( 1+strlen(MyName)+1 );
+	sprintf( DefaultMyStyle, "*%s", MyName );
+    }
+    return DefaultMyStyle ;
+}
+
+char* pixel2string( unsigned long pixel, char* old_string )
+{
+  XColor xcol ;
+    xcol.pixel = pixel ;
+    XQueryColor( Xdisplay, DefaultColormap( Xdisplay, Xscreen), &xcol );
+    if( xcol.flags != 0 )
+    {
+      old_string = realloc( old_string, 1+2+2+2+1 );
+      sprintf( old_string, "#%2.2X%2.2X%2.2X", (xcol.red)>>8, xcol.green>>8, xcol.blue>>8 );
+    }
+    return old_string ;
+}
+
+char* font2string( XFontStruct *font, char* old_string )
+{
+  char* font_name ;
+  Atom name_value ;
+    if(XGetFontProperty(font, XA_FONT, &name_value)) {
+	font_name=XGetAtomName(Xdisplay, name_value);
+	if( font_name )
+	{
+          fprintf( stderr, " font found : [%s]\n", font_name ); 
+	    old_string = realloc( old_string, strlen( font_name)+1 );
+	    strcpy( old_string, font_name );
+	}
+	XFree( font_name );
+    }
+    return old_string ;
+}
+
+
+void set_mystyle( MyStyle* mystyle, Bool reset )
+{
+  int repaint = 0 ;
+  static char* fg_color = NULL, *bg_color = NULL, *font_name = NULL;
+  
+    TermWin.background.mystyle = mystyle ;
+    if( mystyle == NULL )
+    {
+	if( TermWin.background.trgType == BGT_MyStyle )
+	{
+	    if( TermWin.background.trgPixmap != None && 
+		TermWin.background.trgPixmap != TermWin.background.srcPixmap )
+		XFreePixmap( Xdisplay, TermWin.background.trgPixmap );
+	    TermWin.background.trgType = BGT_None ;
+	}
+	return ;
+    }
+    
+    if( mystyle->texture_type > TEXTURE_SOLID && 
+	( !reset
+#ifdef TRANSPARENT
+          || ( !(Options & Opt_transparent) && TermWin.background.srcPixmap == None )
+#endif
+	)    
+      )
+        TermWin.background.trgType = BGT_MyStyle ;
+
+    if( TermWin.background.mystyle )
+    {
+	fprintf( stderr, "using AfterStep MyStyle [%s]:\n", mystyle->name );
+	if(  (mystyle->set_flags & F_FONT) && 
+	    !(TermWin.background.user_flags & F_FONT) )
+	{
+    	    rs_font[0] = font_name = font2string( mystyle->font.font, font_name );
+	    repaint++ ;
+	}
+	if(  (mystyle->set_flags & F_FORECOLOR) &&
+	    !(TermWin.background.user_flags & F_FORECOLOR)  )      
+	{
+	    rs_color[Color_fg] = fg_color = pixel2string(mystyle->colors.fore, fg_color ) ;	    
+	    if( reset )
+	    {
+		PixColorsFocused[Color_fg] = mystyle->colors.fore ;
+#ifdef OFF_FOCUS_FADING
+		PixColorsUnFocused[Color_fg] = fade_color(mystyle->colors.fore);
+#endif
+	    }
+	    repaint++ ;
+	}
+	if(  (mystyle->set_flags & F_BACKCOLOR) &&
+	    !(TermWin.background.user_flags & F_BACKCOLOR)  )      
+	{
+	    rs_color[Color_bg] = bg_color = pixel2string(mystyle->colors.back, bg_color ) ;
+	    if( reset )
+	    {
+		PixColorsFocused[Color_bg] = mystyle->colors.back ;
+#ifdef OFF_FOCUS_FADING
+		PixColorsUnFocused[Color_bg] = fade_color(mystyle->colors.back);
+#endif
+	    }
+	    repaint++ ;
+	}
+	/*if( mystyle->set_flags & F_BACKPIXMAP )      */
+    
+        if( reset )
+	{   if( repaint > 0 )
+	    { /* we want to repaint ourselves */
+		on_colors_changed( Color_bg );
+		if(  (mystyle->set_flags & F_FONT) && 
+		    !(TermWin.background.user_flags & F_FONT)  )
+		    change_font( False, rs_font[0] );
+	    }
+	    if( mystyle->texture_type > TEXTURE_SOLID )
+	    {
+		RenderPixmap(True);
+		scr_clear();
+    		scr_touch();	
+	    }
+	}
+    }    
+}
+#endif
+
 /*{{{ main() */
 /* PROTO */
 int
@@ -1350,7 +1553,16 @@ main(int argc, char *argv[])
     int             saved_argc = argc;
     char          **saved_argv = (char **)MALLOC((argc + 1) * sizeof(char *));
 
+
     PixColors = &(PixColorsFocused[0]);
+
+#ifdef USE_LIBASIMAGE
+    {
+      char *temp = strrchr (argv[0], '/');
+      /* Save our program name - for error messages */
+       MyName = temp ? temp + 1 : argv[0];
+    }
+#endif
     
     for (i = 0; i < argc; i++)
 	saved_argv[i] = argv[i];
@@ -1389,7 +1601,9 @@ main(int argc, char *argv[])
     rs_title = NULL;		/* title name for window */
     rs_iconName = NULL;		/* icon name for window */
     rs_geometry = NULL;		/* window geometry */
+    rs_minBufferWidth = NULL;
     rs_saveLines = NULL;	/* scrollback buffer [lines] */
+    rs_borderWidth = NULL;
     rs_modifier = NULL;	/* modifier */
 #if defined (HOTKEY_CTRL) || defined (HOTKEY_META)
 /* recognized when combined with HOTKEY */
@@ -1430,19 +1644,63 @@ main(int argc, char *argv[])
     if ((display_name = getenv("DISPLAY")) == NULL)
 	display_name = ":0";
 
-    get_options(argc, argv);
-
     Xdisplay = XOpenDisplay(display_name);
     
 #ifdef USE_LIBASIMAGE    
     dpy = Xdisplay ;
+    Scr.screen = Xscreen ;
+    Scr.Root = RootWindow (dpy, Scr.screen);
+    Scr.d_depth = DefaultDepth (dpy, Scr.screen);
+    Scr.MyDisplayWidth = DisplayWidth (dpy, Scr.screen);
+    Scr.MyDisplayHeight = DisplayHeight (dpy, Scr.screen);
+    Scr.CurrentDesk = 0;
+#ifdef _MYSTYLE_
+    
 #endif
+#endif
+#if defined(BACKGROUND_IMAGE) || defined(TRANSPARENT) || defined(_MYSTYLE_)   
+    InitBackground();
+#endif    
+      
+    get_options(argc, argv);
 
     if (!Xdisplay) {
 	print_error("can't open display %s", display_name);
 	exit(EXIT_FAILURE);
     }
     extract_resources(Xdisplay, rs_name);
+    
+#ifdef _MYSTYLE_
+    InternUsefulAtoms();
+    mystyle_get_property(dpy, Scr.Root, _AS_STYLE, XA_INTEGER );
+    /* lets see what options user chosen to override with other 
+       command line parameters : */    
+    if( rs_font[0] != NULL ) 
+	TermWin.background.user_flags |= F_FONT ;
+    if( rs_color[Color_fg] != NULL ) 
+	TermWin.background.user_flags |= F_FORECOLOR ;
+    if( rs_color[Color_bg] != NULL ) 
+	TermWin.background.user_flags |= F_BACKCOLOR ;
+
+    /* reading everything we can from default MyStyle : */
+    set_mystyle(mystyle_find( GetDefaultMyStyle()), False);
+
+    /* overriding it with user specifyed MyStyle : */
+    if( rs_mystyle )
+    {
+      MyStyle * mystyle = mystyle_find( rs_mystyle );
+        if( mystyle == NULL )
+	    fprintf( stderr, "%s: AfterStep MyStyle \"%s\" not available - check your look file.\n", MyName, rs_mystyle );    	
+	else
+	    set_mystyle( mystyle, False );	    
+    }
+#endif
+
+#if defined(BACKGROUND_IMAGE) || defined(TRANSPARENT)
+    if( rs_shade )   
+        TermWin.background.Shading.shading = atoi(rs_shade);
+    SetBackgroundType( rs_backgroundType );
+#endif
 
 #if defined(XTERM_SCROLLBAR) || defined(NEXT_SCROLLBAR)
     sb_shadow = 0;
@@ -1453,17 +1711,17 @@ main(int argc, char *argv[])
 /*
  * set any defaults not already set
  */
-#if defined(BACKGROUND_IMAGE) || defined(TRANSPARENT)    
-    InitBackground(rs_backgroundType, rs_shade);
-#endif    
-
 
     if (!rs_title)
 	rs_title = rs_name;
     if (!rs_iconName)
 	rs_iconName = rs_title;
+    if (!rs_minBufferWidth || (TermWin.min_bcol = atoi(rs_minBufferWidth)) < 0)
+	TermWin.min_bcol = 1;
     if (!rs_saveLines || (TermWin.saveLines = atoi(rs_saveLines)) < 0)
 	TermWin.saveLines = SAVELINES;
+    if (!rs_borderWidth || (TermWin.borderWidth = atoi(rs_borderWidth)) < 0)
+        TermWin.borderWidth = BORDERWIDTH;
 
 /* no point having a scrollbar without having any scrollback! */
     if (!TermWin.saveLines)
@@ -1555,9 +1813,6 @@ main(int argc, char *argv[])
     color_aliases(Color_UL);
 #endif				/* NO_BOLDUNDERLINE */
 
-#ifdef USE_LIBASIMAGE    
-    Scr.screen = Xscreen ;
-#endif
 #ifdef PREFER_24BIT
 
     Xdepth = DefaultDepth(Xdisplay, Xscreen);
@@ -1603,6 +1858,16 @@ main(int argc, char *argv[])
     XMapWindow(Xdisplay, TermWin.vt);
     XMapWindow(Xdisplay, TermWin.parent);
 
+#if defined(BACKGROUND_IMAGE) || defined(TRANSPARENT) || defined(_MYSTYLE_)
+    if( TermWin.background.trgType != BGT_None )
+    {
+        refresh_transparent_scrollbar();
+        RenderPixmap(1);
+	scr_clear();
+	scr_touch();	
+    }
+#endif
+
 #ifdef DEBUG_X
     XSynchronize(Xdisplay, True);
     XSetErrorHandler((XErrorHandler) abort);
