@@ -24,7 +24,7 @@
  *----------------------------------------------------------------------*/
 
 #ifndef lint
-static const char rcsid[] = "$Id: pixmap.c,v 1.13 1998/09/20 09:19:05 mason Exp $";
+static const char rcsid[] = "$Id: pixmap.c,v 1.10 2000/07/01 02:22:19 sashav Exp $";
 #endif
 
 #include "rxvt.h"		/* NECESSARY */
@@ -45,7 +45,7 @@ static const char rcsid[] = "$Id: pixmap.c,v 1.13 1998/09/20 09:19:05 mason Exp
 #endif
 
 
-#if defined(BACKGROUND_IMAGE) || defined(TRANSPARENT)
+#if defined(BACKGROUND_IMAGE) || defined(TRANSPARENT) || defined(_MYSTYLE_)
 /* Xdisplay, Xroot, Xvisual and Xdepth needs to be defined if code is moved to some other prog */
 #define CREATE_TRG_PIXMAP(w,h) XCreatePixmap(Xdisplay, Xroot, w, h, Xdepth)
 
@@ -53,15 +53,30 @@ static const char rcsid[] = "$Id: pixmap.c,v 1.13 1998/09/20 09:19:05 mason Exp
 XImage * ScaleXImageToSize (XImage *src, Position width, Position height );
 void ShadeXImage( XImage* srcImage, ShadingInfo* shading, GC gc );
 
+int
+pixmap_error_handler (Display * dpy, XErrorEvent * error)
+{
+#ifdef DEBUG_IMAGING
+  fprintf(stderr, "\n aterm caused XError # %u, in resource %lu, Request: %d.%d",
+          error->error_code, error->resourceid, error->request_code, error->minor_code );
+#endif	  
+  return 0;
+}
 
+#ifndef LIBASIMAGE_HEADERS
 void CopyAndShadeArea( Drawable src, Pixmap trg, 
                        int x, int y, int w, int h, 
 		       int trg_x, int trg_y, 
 		       GC gc, ShadingInfo* shading )
 {
+  int (*oldXErrorHandler) (Display *, XErrorEvent *) ;
+   /* we need to check if pixmap is still valid */
+    oldXErrorHandler = XSetErrorHandler (pixmap_error_handler);
+
     if( shading )
     {
       XImage* img ;
+
         if( x <0 || y <0  ) return ;
         if((img = XGetImage (Xdisplay, src, x, y, w, h, AllPlanes, ZPixmap))!= NULL )
         {
@@ -71,9 +86,37 @@ void CopyAndShadeArea( Drawable src, Pixmap trg,
 	    return ;
 	}
     }
-    XCopyArea (Xdisplay, src, trg, gc, x, y, w, h, trg_x, trg_y);			
+    if( !XCopyArea (Xdisplay, src, trg, gc, x, y, w, h, trg_x, trg_y)) 
+	XFillRectangle( Xdisplay, trg, gc, trg_x, trg_y, w, h );			
+    XSetErrorHandler (oldXErrorHandler);
 }    
 
+int
+FillPixmapWithTile (Pixmap pixmap, Pixmap tile, int x, int y, int width,
+		    int height, int tile_x, int tile_y)
+{
+  if (tile != None && pixmap != None)
+    {
+      GC gc;
+      XGCValues gcv;
+
+      gcv.tile = tile;
+      gcv.fill_style = FillTiled;
+      gcv.ts_x_origin = -tile_x;
+      gcv.ts_y_origin = -tile_y;
+      gc =
+	XCreateGC (Xdisplay, tile,
+		   GCFillStyle | GCTile | GCTileStipXOrigin |
+		   GCTileStipYOrigin, &gcv);
+      XFillRectangle (Xdisplay, pixmap, gc, x, y, width, height);
+      XFreeGC (Xdisplay, gc);
+      return 1;
+    }
+  return 0;
+}
+
+
+
 void ShadeTiledPixmap(Pixmap src, Pixmap trg, int src_w, int src_h, int x, int y, int w, int h, GC gc, ShadingInfo* shading)
 {
   int tile_x, tile_y, left_w, bott_h;
@@ -202,15 +245,16 @@ GetRootDimensions( int* width, int* height )
 }
 
 int 
-GetWinPosition( Window w, int* x, int* y )
+GetWinPosition (Window win, int *x, int *y)
 {
   Window root, parent, *children ;
   unsigned int nchildren ;
   static int rootWidth = 0, rootHeight = 0;
   XWindowAttributes attr ;
-  int my_x, my_y ;
+  int my_x, my_y, dumm ;
+  unsigned int w, h, udumm ;
 
-    XGetWindowAttributes( Xdisplay, w, &attr );
+    XGetWindowAttributes( Xdisplay, win, &attr );
     if( attr.map_state != IsViewable ) return 0;
 
     if( !x ) x = &my_x ;
@@ -221,112 +265,210 @@ GetWinPosition( Window w, int* x, int* y )
     
     if( !rootWidth || !rootHeight ) 
         if( !GetRootDimensions(&rootWidth, &rootHeight)) return 0;
-          
-    while( XQueryTree( Xdisplay, w, &root, &parent, &children, &nchildren ) )
+
+    XGetGeometry( Xdisplay, win, &root, &dumm, &dumm, &w, &h, &udumm, &udumm );
+  
+    while( XQueryTree( Xdisplay, win, &root, &parent, &children, &nchildren ) )
     {
       int w_x, w_y ;
-      unsigned int w_w, w_h, border_w, w_depth ; 
+      unsigned int border_w ; 
 	if( children ) XFree( children );
-	if( !XGetGeometry( Xdisplay, w, &root, 
-	                  &w_x, &w_y, &w_w, &w_h, &border_w, &w_depth ))	 
+	if( !XGetGeometry( Xdisplay, win, &root, &w_x, &w_y, &udumm, &udumm, &border_w, &udumm ))	 
 	    break ;
-	(*x)+=w_x+border_w ;
-	(*y)+=w_y+border_w ;	
+	(*x)+=w_x+(int)border_w ;
+	(*y)+=w_y+(int)border_w ;	
 	
 	if( parent == root )
 	{/* taking in to consideration virtual desktopping */
-	  int bRes = 0 ;
-	    if( (0<*x && *x<rootWidth)|| ( 0<(*x+w_w) && (*x+w_w)<=rootWidth) ||
-	        (0<*y && *y<=rootHeight ) || ( 0<(*y+w_h) && (*y+w_h)<=rootHeight) ) 
-		bRes = 1 ;
+	  int bRes = 1 ;
+	    if( *x>=rootWidth  || (*x+(int)w) <= 0 ||
+	        *y>=rootHeight || (*y+(int)h) <= 0 ) 
+		bRes = 0 ;
     /* don't want to return position outside the screen even if we fail */
-	    while( *x < 0 ) *x += rootWidth ;
+    /*	    while( *x < 0 ) *x += rootWidth ;
 	    while( *y < 0 ) *y += rootHeight ;
-	    if( *x > rootWidth )   *x %= rootWidth ;
-	    if( *y > rootHeight )  *y %= rootHeight ; 
-	    return bRes; 
+	    while( *x+(int)w > rootWidth ) *x -= rootWidth ;
+	    while( *y+(int)h > rootHeight) *y -= rootHeight ;
+     */	    return bRes; 
 	}
-	w = parent ;
+	win = parent ;
     }
     *x = 0 ;
     *y = 0 ;
     return 0 ;
 }
 
-
-Pixmap 
-CutWinPixmap( Window win, Drawable src, int src_w, int src_h, int width, int height, GC gc, ShadingInfo*  shading)
+void
+sleep_a_little (int n)
 {
-  unsigned int x = 0, y = 0, w, h;
-  Pixmap trg = None; int bNeedToRemap = 0 ;
-/*fprintf(stderr, "\n aterm CutPixmap(): src pixmap is [%lu]", src);  */
+  struct timeval value;
 
-    if( src == None )
-    {
-#ifdef TRANSPARENT
-	if(Options & Opt_transparent)
-	{
-	  Window root, *children ;
-	  unsigned int border, depth, nchildren ;
-	    if( !GetWinPosition(win, NULL,NULL)) return None ;
-    	    if( !XGetGeometry(Xdisplay, win,
-    		 &root, &x, &y, &w, &h, &border, &depth)) return None;
-	    x+=border ;
-	    y+=border ;	    
-	    w-=border*2+1;
-	    h-=border*2+1;
-	    src_w = w+x ;
-	    src_h = h+y ;
-            /* of course we can do withou it, but to make code more generic */
-	    if( !XQueryTree( Xdisplay, win, &root, &src, &children, &nchildren ))
-	        src = None ; /* just in case */
-	    else if( children ) XFree( children );
-	    if( src == None ) return None ;/* it aint gonna happen, coz we always 
-	                                      have root window but just in case */
-	    XUnmapWindow( Xdisplay, win );
-	    bNeedToRemap = 1 ;
-	}
-	else
-#endif  
-	    return None;
-  
-    }else
-    {
-	/* find out our coordinates relative to the root window */
-	if( !GetWinPosition( win, &x, &y ) ) return None ;
-        w = width;
-	h = height;
+  if (n <= 0)
+    return;
 
-        if( x+w > src_w || y+h > src_h )
-	{ /* tiled pixmap processing here */
-	
-	    if((trg = CREATE_TRG_PIXMAP(min(w,src_w), min(h, src_h))))
-		ShadeTiledPixmap(src, trg, src_w, src_h, x, y, min(w,src_w), min(h, src_h), gc, shading); 
-	    return trg ;
-	}
-    }
-    if( x >= 0 && y >= 0 && w >= 2 && h >= 2 )
-    {
+  value.tv_usec = n % 1000000;
+  value.tv_sec = n / 1000000;
 
-	/* create target pixmap of the size of the window */
-	trg = CREATE_TRG_PIXMAP(width, height);
-	if( trg != None )
-	{
-    	/* cut area */
-	/* fprintf( stderr, "\n aterm CutPixmap(): shading src pixmap region [%dx%d+%d+%d]\n source image is %dx%d, window size is %dx%d", 
-	         w, h, x, y, src_w, src_h, width, height);   
-	    fprintf( stderr, "\n aterm: CutPixmap() : trg = %ld\n", trg);			 
-	 */	
-	    CopyAndShadeArea( src, trg, x, y, w, h, 0, 0, gc, shading);			
-
-	}
-    }
-    if( bNeedToRemap )
-        XMapWindow( Xdisplay, win );
-
-    return trg ;
+  (void) select (1, 0, 0, 0, &value);
 }
 
+
+static Pixmap
+CutPixmap ( Pixmap src, Pixmap trg,
+            int x, int y, 
+	    unsigned int src_w, unsigned int src_h, 
+	    unsigned int width, unsigned int height, 
+	    GC gc, ShadingInfo * shading)
+{
+  Bool my_pixmap = (trg == None )?True:False ;
+  int screen_w, screen_h ;
+  int offset_x = 0, offset_y = 0 ;
+  
+  screen_w = DisplayWidth( Xdisplay, Xscreen );   
+  screen_h = DisplayHeight( Xdisplay, Xscreen );
+
+  while( x+(int)width < 0 )  x+= screen_w ;
+  while( x >= screen_w )  x-= screen_w ;
+  while( y+(int)height < 0 )  y+= screen_h ;
+  while( y >= screen_h )  y-= screen_h ;
+
+  if (width < 2 || height < 2 )
+    return trg;
+  if( x < 0 )    
+  {
+    offset_x = (-x) ;
+    x = 0 ; 
+    width -= offset_x ;
+  }
+  if( y < 0 )    
+  {
+    offset_y = (-y) ;
+    y = 0 ;
+    height -= offset_y ;
+  }
+  if( x+width >= screen_w ) width = screen_w - x ;
+  if( y+height >= screen_h ) height = screen_h - y ;    
+  
+  if (src == None) /* we don't have root pixmap ID */
+    { /* we want to create Overrideredirect window overlapping out window
+         with background type of Parent Relative and then grab it */
+     XSetWindowAttributes attr ;
+     XEvent event ;
+     int tick_count = 0 ;
+     Bool grabbed = False ;
+        attr.background_pixmap = ParentRelative ;
+	attr.backing_store = Always ;
+	attr.event_mask = ExposureMask ;
+	attr.override_redirect = True ;
+	src = XCreateWindow(Xdisplay, Xroot, x, y, width, height, 
+	                    0, 
+			    CopyFromParent, CopyFromParent, CopyFromParent,
+			    CWBackPixmap|CWBackingStore|CWOverrideRedirect|CWEventMask,
+			    &attr);
+        
+	if( src == None ) return trg ;
+	XGrabServer( Xdisplay );
+	grabbed = True ;
+	XMapRaised( Xdisplay, src );
+	XSync(Xdisplay, False );
+	/* now we have to wait for our window to become mapped - waiting for Expose */			    
+	for( tick_count = 0 ; !XCheckWindowEvent( Xdisplay, src, ExposureMask, &event ) && tick_count < 100 ; tick_count++)
+	    sleep_a_little(100);
+	    
+	if( tick_count < 100 )	
+	{
+	    if( trg == None )    trg = CREATE_TRG_PIXMAP (width+offset_x, height+offset_y);
+	    if (trg != None)
+	    {	/* custom code to cut area, so to ungrab server ASAP */
+	        if (shading)
+	        {
+	          XImage *img;
+		  img = XGetImage (Xdisplay, src, 0, 0, width, height, AllPlanes, ZPixmap);
+    		  XDestroyWindow( Xdisplay, src );
+	          src = None ;
+		  XUngrabServer( Xdisplay );
+		  grabbed = False ;
+		  if (img != NULL)
+		  {
+    		    ShadeXImage (img, shading, gc);
+		    XPutImage (Xdisplay, trg, gc, img, 0, 0, offset_x, offset_y, width, height);
+		    XDestroyImage (img);
+		  }else if( my_pixmap )
+		  {
+		    
+		    XFreePixmap( Xdisplay, trg );
+		    trg = None ;
+		  }
+		}else
+		    XCopyArea (Xdisplay, src, trg, gc, 0, 0, width, height, offset_x, offset_y);
+	    }
+        }
+	    
+	if( src ) 
+	    XDestroyWindow( Xdisplay, src );
+	if( grabbed )
+	    XUngrabServer( Xdisplay );
+	return trg ;
+    }
+  /* we have root pixmap ID */    
+  /* find out our coordinates relative to the root window */
+  if (x + width > src_w || y + height > src_h)
+    {			/* tiled pixmap processing here */
+      Pixmap tmp ;
+      width = min (width, src_w);
+      height = min (height, src_h);
+      
+      tmp = CREATE_TRG_PIXMAP (width, height);
+      if (tmp != None)
+      {
+        ShadeTiledPixmap (src, tmp, src_w, src_h, x, y, width,
+			  height, gc, shading);
+        if( trg == None )    
+           trg = CREATE_TRG_PIXMAP (width+offset_x, height+offset_y);
+	if( trg != None )
+	    XCopyArea (Xdisplay, tmp, trg, gc, 0, 0, width, height, offset_x, offset_y);
+
+	XFreePixmap( Xdisplay, tmp );
+        return trg;
+      }
+    }
+
+  /* create target pixmap of the size of the window */
+  if( trg == None )    trg = CREATE_TRG_PIXMAP (width+offset_x, height+offset_y);
+  if (trg != None)
+    {
+      /* cut area */
+      CopyAndShadeArea (src, trg, x, y, width, height, offset_x, offset_y, gc, shading);
+
+    }
+
+  return trg;
+}
+
+Pixmap
+CutWinPixmap (Window win, Drawable src, int src_w, int src_h, int width,
+	      int height, GC gc, ShadingInfo * shading)
+{
+  unsigned int x = 0, y = 0;
+
+
+  if( src == None )
+  {
+#ifndef TRANSPARENT  
+    return None ;
+#else
+    if( !(Options & Opt_transparent)) 
+	return None ;
+#endif
+  }
+  
+  if (!GetWinPosition (win, &x, &y))
+	return None;
+
+  return CutPixmap( src, None, x, y, src_w, src_h, width, height, gc, shading );
+}
+
+
 /* PROTO */
 Pixmap 
 GetRootPixmap(Atom id)
@@ -357,16 +499,6 @@ GetRootPixmap(Atom id)
     return currentRootPixmap ;
 }    
 
-int
-pixmap_error_handler (Display * dpy, XErrorEvent * error)
-{
-#ifdef DEBUG_IMAGING
-  fprintf(stderr, "\n aterm caused XError # %u, in resource %lu, Request: %d.%d",
-          error->error_code, error->resourceid, error->request_code, error->minor_code );
-#endif	  
-  return 0;
-}
-
 
 Pixmap
 ValidatePixmap(Pixmap p, int bSetHandler, int bTransparent, unsigned int *pWidth, unsigned int *pHeight)
@@ -393,7 +525,7 @@ ValidatePixmap(Pixmap p, int bSetHandler, int bTransparent, unsigned int *pWidth
     return p ;
 }
 
-
+#endif /* LIBASIMAGE_HEADERS */
 /***************************************************************************/
 /*     Down below goes aterm specific functions                            */
 /***************************************************************************/
@@ -473,6 +605,32 @@ ValidateSrcPixmap(int bSetHandler)
     }
 }
 
+#ifdef _MYSTYLE_
+Pixmap 
+RenderMyStylePixmap( MyStyle *style, Pixmap root_pmap, 
+		     unsigned int root_pmap_width, unsigned int root_pmap_height, 
+		     unsigned int width, unsigned int height )
+{
+/*  fprintf( stderr, "Entering RenderMyStylePixmap : texture_type = %d\n", style->texture_type );
+*/
+  if (style->texture_type == TEXTURE_SOLID)
+    return None;
+  else if (style->texture_type < TEXTURE_PIXMAP)   /* gradients */
+    return mystyle_make_pixmap (style, width, height, None);
+  else if (style->texture_type == TEXTURE_PIXMAP)
+    return style->back_icon.pix;
+  else if (style->texture_type > TEXTURE_PIXMAP)
+    {
+     int real_x, real_y ;
+        GetMyPosition( &real_x, &real_y);
+	return mystyle_make_pixmap_overlay (style, real_x, real_y,
+	    				    width,
+					    height, None);
+    }
+  return None ;
+}
+#endif
+
 /* PROTO */
 void
 RenderPixmap(int DontCheckSource )
@@ -487,7 +645,8 @@ RenderPixmap(int DontCheckSource )
     /* for convinience only */
     fin_width = width ;
     fin_height = height ;
-/*fprintf(stderr, "\n aterm: entering RenderPixmap, window size is %dx%d", width, height);    */
+/*fprintf(stderr, "\n aterm: entering RenderPixmap, window size is %dx%d, trg_type = %d", width, height, BG.trgType );    
+*/
     gcvalue.foreground = PixColors[Color_bg];
     gc = XCreateGC(Xdisplay, TermWin.vt, GCForeground, &gcvalue);
     
@@ -497,7 +656,12 @@ RenderPixmap(int DontCheckSource )
     
     if( !DontCheckSource )  ValidateSrcPixmap( 0 );
     
-    if( BG.srcPixmap == None )
+    
+    if( BG.srcPixmap == None 
+#ifdef _MYSTYLE_
+	&& BG.trgType != BGT_MyStyle
+#endif
+      )
     {
 #ifdef TRANSPARENT
 	if(!(Options & Opt_transparent) || BG.trgType == BGT_None)
@@ -508,8 +672,10 @@ RenderPixmap(int DontCheckSource )
 	}
     }
     
-/*fprintf(stderr, "\n aterm RenderPixmap(): freeing target pixmap ..."); */
+/*fprintf(stderr, "\n aterm RenderPixmap(): freeing target pixmap ..."); 
+*/
     if( BG.trgPixmap != BG.srcPixmap )	FreeTargetPixmap();		
+    
 
     switch( BG.trgType )
     { 
@@ -599,6 +765,18 @@ RenderPixmap(int DontCheckSource )
 		                          width, height, gc, 
 					  SHADING );
 		break ;		
+#ifdef _MYSTYLE_
+	case BGT_MyStyle :
+		BG.trgPixmap = RenderMyStylePixmap( BG.mystyle, BG.srcPixmap, 
+					            BG.Width, BG.Height, 
+		                                    width, height );
+		if( BG.mystyle->texture_type == TEXTURE_PIXMAP)
+		{
+		    BG.srcPixmap = BG.mystyle->back_icon.pix ; /* so not to free it later */
+		    BG.bMySource = False ;
+		}
+		break ;
+#endif
     }
 
     XFreeGC(Xdisplay, gc); /* don't need anymore */
