@@ -1,5 +1,5 @@
 /*
- * $Id: rxvt.h,v 1.19 1998/09/24 17:48:33 mason Exp $
+ * $Id: rxvt.h,v 1.10 2000/07/02 21:08:24 sashav Exp $
  */
 
 #ifndef _RXVT_H			/* include once only */
@@ -141,6 +141,8 @@
 #include <X11/cursorfont.h>
 #include <X11/keysym.h>
 
+#include "libafterstep.h"
+
 #ifdef RXVT_GRAPHICS
 # include "rxvtgrx.h"
 #endif
@@ -188,19 +190,23 @@ typedef RUINTPT		R_u_int_p_t;
 # undef KEEP_SCROLLCOLOR
 #endif
 
-#if defined(BACKGROUND_IMAGE) || defined(TRANSPARENT)
-typedef struct _ShadingInfo{
+#if defined(BACKGROUND_IMAGE) || defined(TRANSPARENT) || defined(_MYSTYLE_)
+
+typedef struct ShadingInfo{
     XColor      tintColor ;
     unsigned int shading ;	
 }ShadingInfo ;
 
 #define NO_NEED_TO_SHADE(o) (o.shading==100 && o.tintColor.red==0xFFFF && o.tintColor.green==0xFFFF && o.tintColor.blue == 0xFFFF)
 #define INIT_SHADING(o)     {o.shading=100; o.tintColor.red=0xFFFF; o.tintColor.green=0xFFFF; o.tintColor.blue = 0xFFFF;}
+struct MyStyle;
 
 typedef struct _BackroundInfo {
     Pixmap	srcPixmap; /* after it was clipped out according to stuff below*/
     /* x,y,width and height of the subimage to be clipped out of
        the specifyed image and used as background */
+    struct MyStyle *mystyle ;   
+    unsigned int user_flags ;
     int 	srcWidth, srcHeight, srcX, srcY ;
     int 	Width, Height ; /* width and height of the loaded pixmap */
     int		bMySource;/* indicates if src pixmap was loaded by us
@@ -224,6 +230,9 @@ typedef struct _BackroundInfo {
 	   BGT_NoTileH,
 	   BGT_NoTileV,
 	   BGT_Cut,
+#ifdef _MYSTYLE_
+	   BGT_MyStyle,
+#endif
 	   BGT_None } trgType;
 	   
 
@@ -250,8 +259,11 @@ typedef struct {
 		    fheight,	/* font height [pixels]                     */
 		    fprop,	/* font is proportional                     */
 		    ncol, nrow,	/* window size [characters]                 */
+		    bcol,	/* current number of columns in the buffer  */
+		    min_bcol,	/* minimum horizontal columns in the buffer */
 		    focus,	/* window has focus                         */
 		    saveLines,	/* number of lines that fit in scrollback   */
+		    borderWidth,/* number of pixels in window border        */
 		    nscrolled,	/* number of line actually scrolled         */
 		    view_start;	/* scrollback view starts here              */
     Window          parent, vt;	/* parent (main) and vt100 window           */
@@ -263,7 +275,7 @@ typedef struct {
 #ifdef MULTICHAR_SET
     XFontStruct    *mfont;	/* Multichar font structure                 */
 #endif
-#if defined(BACKGROUND_IMAGE) || defined(TRANSPARENT)
+#if defined(BACKGROUND_IMAGE) || defined(TRANSPARENT) || defined(_MYSTYLE_)
     BackgroundInfo background;
 #endif
 
@@ -283,7 +295,7 @@ typedef struct {
 #define TINT_TYPE_TRUE	"true"
 #define TINT_TYPE_ALL 	"true|and|andReverse|andInverted|xor|or|nor|invert|equiv|invert|orReverse|orInverted|nand"
 
-#if defined(BACKGROUND_IMAGE) || defined(TRANSPARENT)
+#if defined(BACKGROUND_IMAGE) || defined(TRANSPARENT) || defined(_MYSTYLE_)
     GC              tintGC;
     Pixmap	    LastPixmapUsed ;
 #endif
@@ -532,8 +544,17 @@ typedef struct {
 #ifdef WTMP_SUPPORT
 # ifdef HAVE_UTMPX_H		/* means we're on Solaris (or Irix?) */
 #  undef WTMP_FILENAME
-#  define WTMP_FILENAME		WTMPX_FILE
-#  define update_wtmp		updwtmpx
+#  if  defined(WTMPX_FILE) || defined(_WTMPX_FILE)
+#    if defined(WTMPX_FILE)
+#    	define WTMP_FILENAME	WTMPX_FILE
+#    else
+#    	define WTMP_FILENAME	_WTMPX_FILE
+#    endif
+#    define update_wtmp		updwtmpx
+#  else
+#       define WTMP_FILENAME	RXVT_WTMP_FILE
+#       define update_wtmp	rxvt_update_wtmp
+#  endif
 # else
 #  define update_wtmp		rxvt_update_wtmp
 #  ifndef WTMP_FILENAME
@@ -836,7 +857,7 @@ enum colour_list {
 # define menubar_visible()	(0)
 #endif
 
-#ifdef BACKGROUND_IMAGE
+#if defined(BACKGROUND_IMAGE) || defined(_MYSTYLE_)
 # define XPMClearArea(a, b, c, d, e, f, g)	XClearArea((a), (b), (c), (d), (e), (f), (g))
 #else
 # define XPMClearArea(a, b, c, d, e, f, g)
@@ -889,11 +910,16 @@ EXTERN Pixel		PixColorsFocused[TOTAL_COLORS];
 #ifdef OFF_FOCUS_FADING
 EXTERN Pixel		PixColorsUnFocused[TOTAL_COLORS];
 #endif
+#ifdef _MYSTYLE_
+EXTERN const char      *rs_mystyle;
+#endif
 EXTERN Pixel	       *PixColors;
 EXTERN const char      *rs_title;	/* title name for window */
 EXTERN const char      *rs_iconName;	/* icon name for window */
 EXTERN const char      *rs_geometry;	/* window geometry */
+EXTERN const char      *rs_minBufferWidth; /* minimum buffer width - so we can scroll horizontally */
 EXTERN const char      *rs_saveLines;	/* scrollback buffer [lines] */
+EXTERN const char      *rs_borderWidth; /* border width [pixels] */
 EXTERN const char      *rs_cutchars;	/* chars for selection boundaries */
 #ifdef META8_OPTION
 EXTERN const char      *rs_modifier;
