@@ -1,6 +1,7 @@
 /*--------------------------------*-C-*--------------------------------------*
  * File:      screen.c
  *---------------------------------------------------------------------------*
+ * Copyright (C) 2000 Sasha Vasko <sashav@sprintmail.com>
  * Copyright (C) 1997,1998 Geoff Wing <gcw@pobox.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -32,7 +33,7 @@
  */
 
 #ifndef lint
-static const char rcsid[] = "$Id: screen.c,v 1.29 1998/09/24 09:18:34 mason Exp $";
+static const char rcsid[] = "$Id: screen.c,v 1.5 2000/05/30 22:17:56 sashav Exp $";
 #endif
 
 #define INTERN_SCREEN
@@ -41,9 +42,11 @@ static const char rcsid[] = "$Id: screen.c,v 1.29 1998/09/24 09:18:34 mason Exp
 #include <X11/Xatom.h>
 #include <X11/Xmd.h>		/* get the typedef for CARD32 */
 
-#ifdef OFF_FOCUS_FADING
-int focus_changed = 0 ; /* ised here and in command.c to track focus on-off*/
-#endif
+
+static screen_t    screen;
+
+
+int colors_changed = 0 ; /* to indicate that foreground and background colors has changed*/
 
 /* ------------------------------------------------------------------------- */
 #ifdef MULTICHAR_SET
@@ -206,15 +209,15 @@ blank_line(text_t * et, rend_t * er, int width, rend_t efs)
 void
 make_screen_mem(text_t ** tp, rend_t ** rp, int row, rend_t rstyle)
 {
-    tp[row] = MALLOC(sizeof(text_t) * TermWin.ncol);
-    rp[row] = MALLOC(sizeof(rend_t) * TermWin.ncol);
-    blank_line(tp[row], rp[row], TermWin.ncol, rstyle);
+    tp[row] = MALLOC(sizeof(text_t) * TermWin.bcol);
+    rp[row] = MALLOC(sizeof(rend_t) * TermWin.bcol);
+    blank_line(tp[row], rp[row], TermWin.bcol, rstyle);
 }
 
 /* ------------------------------------------------------------------------- *
  *                          SCREEN INITIALISATION                            *
  * ------------------------------------------------------------------------- */
-static int      prev_nrow = -1, prev_ncol = -1;
+static int      prev_nrow = -1, prev_bcol = -1;
 
 /* PROTO */
 void
@@ -228,13 +231,9 @@ scr_reset(void)
     TermWin.view_start = 0;
     RESET_CHSTAT;
 
-    if (TermWin.ncol == prev_ncol && TermWin.nrow == prev_nrow)
+    if (TermWin.bcol == prev_bcol && TermWin.nrow == prev_nrow)
 	return;
 
-    if (TermWin.ncol <= 0)
-	TermWin.ncol = 80;
-    if (TermWin.nrow <= 0)
-	TermWin.nrow = 24;
 #ifdef DEBUG_STRICT
     assert(TermWin.saveLines >= 0);
 #else				/* drive with your eyes closed */
@@ -269,11 +268,8 @@ scr_reset(void)
 	    j = i + TermWin.saveLines;
 	    make_screen_mem(screen.text, screen.rend, j, DEFAULT_RSTYLE);
 	    make_screen_mem(swap.text, swap.rend, i, DEFAULT_RSTYLE);
+	    make_screen_mem(drawn_text, drawn_rend, i, DEFAULT_RSTYLE );
 	    screen.tlen[j] = swap.tlen[i] = 0;
-	    drawn_text[i] = MALLOC(TermWin.ncol * sizeof(text_t));
-	    drawn_rend[i] = MALLOC(TermWin.ncol * sizeof(rend_t));
-	    blank_line(drawn_text[i], drawn_rend[i], TermWin.ncol,
-		       DEFAULT_RSTYLE);
 	}
 	TermWin.nscrolled = 0;	/* no saved lines */
 	screen.flags = swap.flags = Screen_DefaultFlags;
@@ -323,25 +319,11 @@ scr_reset(void)
 		FREE(drawn_text[i]);
 		FREE(drawn_rend[i]);
 	    }
-	    screen.text = REALLOC(screen.text, total_rows  *sizeof(text_t*));
-	    screen.tlen = REALLOC(screen.tlen, total_rows  *sizeof(R_int16_t));
-	    buf_text  	= REALLOC(buf_text   , total_rows  *sizeof(text_t*));
-	    buf_tlen  	= REALLOC(buf_tlen   , total_rows  *sizeof(R_int16_t));
-	    drawn_text  = REALLOC(drawn_text , TermWin.nrow*sizeof(text_t*));
-	    swap.text  	= REALLOC(swap.text  , TermWin.nrow*sizeof(text_t*));
-	    swap.tlen   = REALLOC(swap.tlen  , total_rows  *sizeof(R_int16_t));
-
-	    screen.rend = REALLOC(screen.rend, total_rows  *sizeof(rend_t*));
-	    buf_rend  	= REALLOC(buf_rend   , total_rows  *sizeof(rend_t*));
-	    drawn_rend  = REALLOC(drawn_rend , TermWin.nrow*sizeof(rend_t*));
-	    swap.rend  	= REALLOC(swap.rend  , TermWin.nrow*sizeof(rend_t*));
-
 	/* we have fewer rows so fix up number of scrolled lines */
 	    MIN_IT(screen.cur.row, TermWin.nrow - 1);
 	    MIN_IT(swap.cur.row, TermWin.nrow - 1);
-
-	} else if (TermWin.nrow > prev_nrow) {
-	/* add rows */
+	}
+	if(TermWin.nrow != prev_nrow) {
 	    screen.text = REALLOC(screen.text, total_rows  *sizeof(text_t*));
 	    screen.tlen = REALLOC(screen.tlen, total_rows  *sizeof(R_int16_t));
 	    buf_text  	= REALLOC(buf_text   , total_rows  *sizeof(text_t*));
@@ -349,12 +331,14 @@ scr_reset(void)
 	    drawn_text  = REALLOC(drawn_text , TermWin.nrow*sizeof(text_t*));
 	    swap.text  	= REALLOC(swap.text  , TermWin.nrow*sizeof(text_t*));
 	    swap.tlen   = REALLOC(swap.tlen  , total_rows  *sizeof(R_int16_t));
-			  
+
 	    screen.rend = REALLOC(screen.rend, total_rows  *sizeof(rend_t*));
 	    buf_rend  	= REALLOC(buf_rend   , total_rows  *sizeof(rend_t*));
 	    drawn_rend  = REALLOC(drawn_rend , TermWin.nrow*sizeof(rend_t*));
 	    swap.rend  	= REALLOC(swap.rend  , TermWin.nrow*sizeof(rend_t*));
-
+	}
+	if (TermWin.nrow > prev_nrow) {
+	/* add rows */
 	    k = min(TermWin.nscrolled, TermWin.nrow - prev_nrow);
 	    for (i = prev_total_rows; i < total_rows - k; i++) {
 		make_screen_mem(screen.text, screen.rend, i, setrstyle);
@@ -367,14 +351,10 @@ scr_reset(void)
 	    }
 	    for (i = prev_nrow; i < TermWin.nrow; i++) {
 		make_screen_mem(swap.text, swap.rend, i, setrstyle);
+	        make_screen_mem(drawn_text, drawn_rend, i, setrstyle );
 		swap.tlen[i] = 0;
 	    }
-	    for (i = prev_nrow; i < TermWin.nrow; i++) {
-		drawn_text[i]  = MALLOC(TermWin.ncol * sizeof(text_t));
-		drawn_rend[i]  = MALLOC(TermWin.ncol * sizeof(rend_t));
-		blank_line(drawn_text[i], drawn_rend[i], TermWin.ncol,
-			   setrstyle);
-	    }
+
 	    if (k > 0) {
 		scroll_text(0, TermWin.nrow - 1, -k, 1);
 		screen.cur.row += k;
@@ -396,56 +376,56 @@ scr_reset(void)
 	}
 
 /* B2: resize columns */
-	if (TermWin.ncol != prev_ncol) {
+	if (TermWin.bcol != prev_bcol) {
 	    for (i = 0; i < total_rows; i++) {
 		if (screen.text[i]) {
 		    screen.text[i] = REALLOC(screen.text[i],
-					      TermWin.ncol * sizeof(text_t));
+					      TermWin.bcol * sizeof(text_t));
 		    screen.rend[i] = REALLOC(screen.rend[i],
-					      TermWin.ncol * sizeof(rend_t));
-		    MIN_IT(screen.tlen[i], TermWin.ncol);
-		    if (TermWin.ncol > prev_ncol)
-			blank_line(&(screen.text[i][prev_ncol]),
-				   &(screen.rend[i][prev_ncol]),
-				   TermWin.ncol - prev_ncol, setrstyle);
+					      TermWin.bcol * sizeof(rend_t));
+		    MIN_IT(screen.tlen[i], TermWin.bcol);
+		    if (TermWin.bcol > prev_bcol)
+			blank_line(&(screen.text[i][prev_bcol]),
+				   &(screen.rend[i][prev_bcol]),
+				   TermWin.bcol - prev_bcol, setrstyle);
 		}
 	    }
 	    for (i = 0; i < TermWin.nrow; i++) {
 		drawn_text[i] = REALLOC(drawn_text[i],
-					 TermWin.ncol * sizeof(text_t));
+					 TermWin.bcol * sizeof(text_t));
 		drawn_rend[i] = REALLOC(drawn_rend[i],
-					 TermWin.ncol * sizeof(rend_t));
+					 TermWin.bcol * sizeof(rend_t));
 		if (swap.text[i]) {
 		    swap.text[i] = REALLOC(swap.text[i],
-					    TermWin.ncol * sizeof(text_t));
+					    TermWin.bcol * sizeof(text_t));
 		    swap.rend[i] = REALLOC(swap.rend[i],
-					    TermWin.ncol * sizeof(rend_t));
-		    MIN_IT(swap.tlen[i], TermWin.ncol);
-		    if (TermWin.ncol > prev_ncol)
-			blank_line(&(swap.text[i][prev_ncol]),
-				   &(swap.rend[i][prev_ncol]),
-				   TermWin.ncol - prev_ncol, setrstyle);
+					    TermWin.bcol * sizeof(rend_t));
+		    MIN_IT(swap.tlen[i], TermWin.bcol);
+		    if (TermWin.bcol > prev_bcol)
+			blank_line(&(swap.text[i][prev_bcol]),
+				   &(swap.rend[i][prev_bcol]),
+				   TermWin.bcol - prev_bcol, setrstyle);
 		}
-		if (TermWin.ncol > prev_ncol)
-		    blank_line(&(drawn_text[i][prev_ncol]),
-			       &(drawn_rend[i][prev_ncol]),
-			       TermWin.ncol - prev_ncol, setrstyle);
+		if (TermWin.bcol > prev_bcol)
+		    blank_line(&(drawn_text[i][prev_bcol]),
+			       &(drawn_rend[i][prev_bcol]),
+			       TermWin.bcol - prev_bcol, setrstyle);
 	    }
-	    MIN_IT(screen.cur.col, TermWin.ncol - 1);
-	    MIN_IT(swap.cur.col, TermWin.ncol - 1);
+	    MIN_IT(screen.cur.col, TermWin.bcol - 1);
+	    MIN_IT(swap.cur.col, TermWin.bcol - 1);
 	}
 	if (tabs)
 	    FREE(tabs);
     }
 /* *INDENT-ON* */
 
-    tabs = MALLOC(TermWin.ncol * sizeof(char));
+    tabs = MALLOC(TermWin.bcol * sizeof(char));
 
-    for (i = 0; i < TermWin.ncol; i++)
+    for (i = 0; i < TermWin.bcol; i++)
 	tabs[i] = (i % TABSIZE == 0) ? 1 : 0;
 
     prev_nrow = TermWin.nrow;
-    prev_ncol = TermWin.ncol;
+    prev_bcol = TermWin.bcol;
 
     tt_resize();
 }
@@ -514,7 +494,7 @@ scr_poweron(void)
 
     scr_release();
     prev_nrow = -1;
-    prev_ncol = -1;
+    prev_bcol = -1;
     scr_reset();
 
     scr_clear();
@@ -555,7 +535,7 @@ scr_cursor(int mode)
     }
 /* boundary check in case screen size changed between SAVE and RESTORE */
     MIN_IT(screen.cur.row, TermWin.nrow - 1);
-    MIN_IT(screen.cur.col, TermWin.ncol - 1);
+    MIN_IT(screen.cur.col, TermWin.bcol - 1);
 #ifdef DEBUG_STRICT
     assert(screen.cur.row >= 0);
     assert(screen.cur.col >= 0);
@@ -604,12 +584,12 @@ scr_change_screen(int scrn)
     assert(screen.cur.row >= 0);
     assert(screen.cur.col >= 0);
     assert(screen.cur.row < TermWin.nrow);
-    assert(screen.cur.col < TermWin.ncol);
+    assert(screen.cur.col < TermWin.bcol);
 # else				/* drive with your eyes closed */
     MAX_IT(screen.cur.row, 0);
     MAX_IT(screen.cur.col, 0);
     MIN_IT(screen.cur.row, TermWin.nrow - 1);
-    MIN_IT(screen.cur.col, TermWin.ncol - 1);
+    MIN_IT(screen.cur.col, TermWin.bcol - 1);
 # endif
     SWAP_IT(screen.charset, swap.charset, l0);
     SWAP_IT(screen.flags, swap.flags, tmp);
@@ -861,7 +841,7 @@ scr_add_lines(const unsigned char *str, int nlines, int len)
     if (len <= 0)		/* sanity */
 	return;
 
-    last_col = TermWin.ncol;
+    last_col = TermWin.bcol;
 
     D_SCREEN((stderr, "scr_add_lines(*,%d,%d)", nlines, len));
     ZERO_SCROLLBACK;
@@ -876,7 +856,7 @@ scr_add_lines(const unsigned char *str, int nlines, int len)
 		if (screen.text[j] == NULL)
 		    make_screen_mem(screen.text, screen.rend, j, rstyle);
 		else
-		    blank_line(screen.text[j], screen.rend[j], TermWin.ncol,
+		    blank_line(screen.text[j], screen.rend[j], TermWin.bcol,
 			       rstyle);
 		screen.tlen[j] = 0;
 	    }
@@ -951,7 +931,7 @@ scr_add_lines(const unsigned char *str, int nlines, int len)
 					    rstyle);
 			else
 			    blank_line(screen.text[j], screen.rend[j],
-				       TermWin.ncol, rstyle);
+				       TermWin.bcol, rstyle);
 			screen.tlen[j] = 0;
 		    } else if (screen.cur.row < (TermWin.nrow - 1))
 			row = (++screen.cur.row) + TermWin.saveLines;
@@ -979,7 +959,7 @@ scr_add_lines(const unsigned char *str, int nlines, int len)
 		if (screen.text[j] == NULL)
 		    make_screen_mem(screen.text, screen.rend, j, rstyle);
 		else
-		    blank_line(screen.text[j], screen.rend[j], TermWin.ncol,
+		    blank_line(screen.text[j], screen.rend[j], TermWin.bcol,
 			       rstyle);
 		screen.tlen[j] = 0;
 	    } else if (screen.cur.row < (TermWin.nrow - 1))
@@ -1041,7 +1021,7 @@ scr_backspace(void)
 {
     RESET_CHSTAT;
     if (screen.cur.col == 0 && screen.cur.row > 0) {
-	screen.cur.col = TermWin.ncol - 1;
+	screen.cur.col = TermWin.bcol - 1;
 	screen.cur.row--;
     } else if (screen.flags & Screen_WrapNext) {
 	screen.flags &= ~Screen_WrapNext;
@@ -1066,7 +1046,7 @@ scr_tab(int count)
     if (count == 0)
 	return;
     else if (count > 0) {
-	for (i = x + 1; i < TermWin.ncol; i++) {
+	for (i = x + 1; i < TermWin.bcol; i++) {
 	    if (tabs[i]) {
 		x = i;
 		if (!--count)
@@ -1103,7 +1083,7 @@ scr_gotorc(int row, int col, int relative)
 
     screen.cur.col = ((relative & C_RELATIVE) ? (screen.cur.col + col) : col);
     MAX_IT(screen.cur.col, 0);
-    MIN_IT(screen.cur.col, TermWin.ncol - 1);
+    MIN_IT(screen.cur.col, TermWin.bcol - 1);
 
     if (screen.flags & Screen_WrapNext) {
 	screen.flags &= ~Screen_WrapNext;
@@ -1164,7 +1144,7 @@ scr_index(int direction)
 	if (screen.text[dirn] == NULL)	/* then so is screen.rend[dirn] */
 	    make_screen_mem(screen.text, screen.rend, dirn, rstyle);
 	else
-	    blank_line(screen.text[dirn], screen.rend[dirn], TermWin.ncol,
+	    blank_line(screen.text[dirn], screen.rend[dirn], TermWin.bcol,
 		       rstyle);
 	screen.tlen[dirn] = 0;
     } else
@@ -1201,7 +1181,7 @@ scr_erase_line(int mode)
     switch (mode) {
     case 0:			/* erase to end of line */
 	col = screen.cur.col;
-	num = TermWin.ncol - col;
+	num = TermWin.bcol - col;
 	MIN_IT(screen.tlen[row], col);
 	if (ROWCOL_IN_ROW_ON_OR_AFTER(selection.beg, screen.cur)
 	    || ROWCOL_IN_ROW_ON_OR_AFTER(selection.end, screen.cur))
@@ -1216,7 +1196,7 @@ scr_erase_line(int mode)
 	break;
     case 2:			/* erase whole line */
 	col = 0;
-	num = TermWin.ncol;
+	num = TermWin.bcol;
 	screen.tlen[row] = 0;
 	if (selection.beg.row <= screen.cur.row
 	    || selection.end.row >= screen.cur.row)
@@ -1296,10 +1276,10 @@ scr_erase_screen(int mode)
 	}
 	for (; num--; row++) {
 	    blank_line(screen.text[row + row_offset],
-		       screen.rend[row + row_offset], TermWin.ncol,
+		       screen.rend[row + row_offset], TermWin.bcol,
 		       rstyle & ~RS_Uline);
 	    screen.tlen[row + row_offset] = 0;
-	    blank_line(drawn_text[row], drawn_rend[row], TermWin.ncol, ren);
+	    blank_line(drawn_text[row], drawn_rend[row], TermWin.bcol, ren);
 	}
     }
 }
@@ -1325,11 +1305,11 @@ scr_E(void)
     for (i = TermWin.saveLines; i < TermWin.nrow + TermWin.saveLines; i++) {
 	t = screen.text[i];
 	r = screen.rend[i];
-	for (j = 0; j < TermWin.ncol; j++) {
+	for (j = 0; j < TermWin.bcol; j++) {
 	    *t++ = 'E';
 	    *r++ = fs;
 	}
-	screen.tlen[i] = TermWin.ncol;	/* make the `E's selectable */
+	screen.tlen[i] = TermWin.bcol;	/* make the `E's selectable */
     }
 }
 
@@ -1374,7 +1354,7 @@ scr_insdel_lines(int count, int insdel)
 	if (screen.text[end] == NULL)	/* then so is screen.rend[end] */
 	    make_screen_mem(screen.text, screen.rend, end, rstyle);
 	else
-	    blank_line(screen.text[end], screen.rend[end], TermWin.ncol,
+	    blank_line(screen.text[end], screen.rend[end], TermWin.bcol,
 		       rstyle);
 	screen.tlen[end] = 0;
     }
@@ -1400,25 +1380,25 @@ scr_insdel_chars(int count, int insdel)
 	return;
 
     CHECK_SELECTION(1);
-    MIN_IT(count, (TermWin.ncol - screen.cur.col));
+    MIN_IT(count, (TermWin.bcol - screen.cur.col));
 
     row = screen.cur.row + TermWin.saveLines;
     screen.flags &= ~Screen_WrapNext;
 
     switch (insdel) {
     case INSERT:
-	for (col = TermWin.ncol - 1; (col - count) >= screen.cur.col; col--) {
+	for (col = TermWin.bcol - 1; (col - count) >= screen.cur.col; col--) {
 	    screen.text[row][col] = screen.text[row][col - count];
 	    screen.rend[row][col] = screen.rend[row][col - count];
 	}
 	if (screen.tlen[row] != -1) {
 	    screen.tlen[row] += count;
-	    MIN_IT(screen.tlen[row], TermWin.ncol);
+	    MIN_IT(screen.tlen[row], TermWin.bcol);
 	}
 	if (selection.op && current_screen == selection.screen
 	    && ROWCOL_IN_ROW_ON_OR_AFTER(selection.beg, screen.cur)) {
 	    if (selection.end.row != screen.cur.row
-		|| (selection.end.col + count >= TermWin.ncol))
+		|| (selection.end.col + count >= TermWin.bcol))
 		CLEAR_SELECTION;
 	    else {		/* shift selection */
 		selection.beg.col += count;
@@ -1431,7 +1411,7 @@ scr_insdel_chars(int count, int insdel)
 		   count, rstyle);
 	break;
     case ERASE:
-	screen.cur.col += count;	/* don't worry if > TermWin.ncol */
+	screen.cur.col += count;	/* don't worry if > TermWin.bcol */
 	CHECK_SELECTION(1);
 	screen.cur.col -= count;
 	blank_line(&(screen.text[row][screen.cur.col]),
@@ -1439,24 +1419,24 @@ scr_insdel_chars(int count, int insdel)
 		   count, rstyle);
 	break;
     case DELETE:
-	tr = screen.rend[row][TermWin.ncol - 1]
+	tr = screen.rend[row][TermWin.bcol - 1]
 	     & (RS_fgMask | RS_bgMask | RS_baseattrMask);
-	for (col = screen.cur.col; (col + count) < TermWin.ncol; col++) {
+	for (col = screen.cur.col; (col + count) < TermWin.bcol; col++) {
 	    screen.text[row][col] = screen.text[row][col + count];
 	    screen.rend[row][col] = screen.rend[row][col + count];
 	}
-	blank_line(&(screen.text[row][TermWin.ncol - count]),
-		   &(screen.rend[row][TermWin.ncol - count]),
+	blank_line(&(screen.text[row][TermWin.bcol - count]),
+		   &(screen.rend[row][TermWin.bcol - count]),
 		   count, tr);
 	if (screen.tlen[row] == -1)	/* break line continuation */
-	    screen.tlen[row] = TermWin.ncol;
+	    screen.tlen[row] = TermWin.bcol;
 	screen.tlen[row] -= count;
 	MAX_IT(screen.tlen[row], 0);
 	if (selection.op && current_screen == selection.screen
 	    && ROWCOL_IN_ROW_ON_OR_AFTER(selection.beg, screen.cur)) {
 	    if (selection.end.row != screen.cur.row
 		|| (screen.cur.col >= selection.beg.col - count)
-		|| selection.end.col >= TermWin.ncol)
+		|| selection.end.col >= TermWin.bcol)
 		CLEAR_SELECTION;
 	    else {
 		/* shift selection */
@@ -1472,9 +1452,9 @@ scr_insdel_chars(int count, int insdel)
 	screen.rend[row][0] &= ~RS_multiMask;
 	screen.text[row][0] = ' ';
     }
-    if ((screen.rend[row][TermWin.ncol - 1] & RS_multiMask) == RS_multi1) {
-	screen.rend[row][TermWin.ncol - 1] &= ~RS_multiMask;
-	screen.text[row][TermWin.ncol - 1] = ' ';
+    if ((screen.rend[row][TermWin.bcol - 1] & RS_multiMask) == RS_multi1) {
+	screen.rend[row][TermWin.bcol - 1] &= ~RS_multiMask;
+	screen.text[row][TermWin.bcol - 1] = ' ';
     }
 #endif
 }
@@ -1578,9 +1558,9 @@ void
 scr_set_tab(int mode)
 {
     if (mode < 0)
-	MEMSET(tabs, 0, TermWin.ncol * sizeof(char));
+	MEMSET(tabs, 0, TermWin.bcol * sizeof(char));
 
-    else if (screen.cur.col < TermWin.ncol)
+    else if (screen.cur.col < TermWin.bcol)
 	tabs[screen.cur.col] = (mode ? 1 : 0);
 }
 
@@ -1602,7 +1582,7 @@ scr_rvideo_mode(int mode)
 
 	maxlines = TermWin.saveLines + TermWin.nrow;
 	for (i = TermWin.saveLines; i < maxlines; i++)
-	    for (j = 0; j < TermWin.ncol + 1; j++)
+	    for (j = 0; j < TermWin.bcol + 1; j++)
 		screen.rend[i][j] ^= RS_RVid;
 	scr_refresh(SLOW_REFRESH);
     }
@@ -1818,10 +1798,10 @@ scr_expose(int x, int y, int width, int height)
     MAX_IT(full_beg.row, 0);
     MAX_IT(part_end.row, 0);
     MAX_IT(full_end.row, 0);
-    MIN_IT(part_beg.col, TermWin.ncol - 1);
-    MIN_IT(full_beg.col, TermWin.ncol - 1);
-    MIN_IT(part_end.col, TermWin.ncol - 1);
-    MIN_IT(full_end.col, TermWin.ncol - 1);
+    MIN_IT(part_beg.col, TermWin.bcol - 1);
+    MIN_IT(full_beg.col, TermWin.bcol - 1);
+    MIN_IT(part_end.col, TermWin.bcol - 1);
+    MIN_IT(full_end.col, TermWin.bcol - 1);
     MIN_IT(part_beg.row, TermWin.nrow - 1);
     MIN_IT(full_beg.row, TermWin.nrow - 1);
     MIN_IT(part_end.row, TermWin.nrow - 1);
@@ -1959,7 +1939,7 @@ scr_printscreen(int fullhist)
     row_offset = TermWin.saveLines - TermWin.view_start;
     for (r = 0; r < nrows; r++) {
 	t = screen.text[r + row_offset];
-	for (i = TermWin.ncol - 1; i >= 0; i--)
+	for (i = TermWin.bcol - 1; i >= 0; i--)
 	    if (!isspace(t[i]))
 		break;
 	fprintf(fd, "%.*s\n", (i + 1), t);
@@ -1984,12 +1964,6 @@ scr_printscreen(int fullhist)
 # define MONO_BOLD(x)	(((x) & RS_Bold) && fore == Color_fg)
 #endif
 
-#define FONT_WIDTH(X, Y)						\
-    (X)->per_char[(Y) - (X)->min_char_or_byte2].width
-#define FONT_RBEAR(X, Y)						\
-    (X)->per_char[(Y) - (X)->min_char_or_byte2].rbearing
-
-
 /* PROTO */
 void
 scr_refresh(int type)
@@ -2059,6 +2033,7 @@ scr_refresh(int type)
     is_cursor = 0;
     gcvalue.foreground = PixColors[Color_fg];
     gcvalue.background = PixColors[Color_bg];
+
 /*
  * always go back to the base font - it's much safer
  */
@@ -2087,6 +2062,7 @@ scr_refresh(int type)
 	    wf = TermWin.font;
 	    j = wbyte;
 	    for (col = TermWin.ncol - 2; col >= 0; col--) {
+	      int do_stuff ;
 # if ! defined (NO_BRIGHTCOLOR) && ! defined (VERYBOLD)
 		fore = GET_FGCOLOR(drp[col]);
 # endif
@@ -2102,14 +2078,26 @@ scr_refresh(int type)
 		if (dtp[col] == ' ') {	/* TODO: check character set? */
 		    continue;
 		}
-		if (wf->per_char == NULL
-		    || FONT_WIDTH(wf, dtp[col]) == FONT_RBEAR(wf, dtp[col])) {
+
+		if( wf->per_char == NULL )
+		    do_stuff = 1 ;
+		else if( dtp[col] >= wf->min_char_or_byte2 &&
+		    dtp[col] <  wf->max_char_or_byte2)
+		{
+		  int char_num = dtp[col]-wf->min_char_or_byte2 ;
+		    do_stuff = (wf->per_char[char_num].width == wf->per_char[char_num].rbearing);
+		}
+		else
+		    do_stuff = 0 ;
+
+		if (do_stuff)
+		{
 		    dtp[col + 1] = 0;
 # if defined(MULTICHAR_SET) && ! defined(NO_BOLDOVERSTRIKE_MULTI)
 		    if ((srp[col] & RS_multiMask) == RS_multi2) {
-			col--;
-			wbyte = 1;
-			continue;
+		        col--;
+		        wbyte = 1;
+		        continue;
 		    }
 # endif
 		}
@@ -2155,7 +2143,7 @@ scr_refresh(int type)
 #ifdef MULTICHAR_SET
 	rt1 = *srp & RS_multiMask;
 	if (rt1 == RS_multi1) {
-	    if (screen.cur.col < TermWin.ncol - 2
+	    if (screen.cur.col < TermWin.bcol - 2
 		&& ((srp[1] & RS_multiMask) == RS_multi2))
 		morecur = 1;
 	} else if (rt1 == RS_multi2) {
@@ -2180,7 +2168,7 @@ scr_refresh(int type)
     /* make sure no outline cursor is left around */
 	if (screen.cur.row != oldcursor.row
 	    || screen.cur.col != oldcursor.col) {
-	    if (oldcursor.row < TermWin.nrow && oldcursor.col < TermWin.ncol) {
+	    if (oldcursor.row < TermWin.nrow && oldcursor.col < TermWin.bcol) {
 		drawn_text[oldcursor.row][oldcursor.col] = 0;
 #ifdef MULTICHAR_SET
 		if (oldcursormulti) {
@@ -2260,13 +2248,13 @@ scr_refresh(int type)
 			draw_image_string = XDrawImageString16;
 		    }
 		/* double stepping - we're in Kanji mode */
-		    for (; ++col < TermWin.ncol;) {
+		    for (; ++col < TermWin.bcol;) {
 		    /* XXX: could check sanity on 2nd byte */
 			dtp[col] = stp[col];
 			drp[col] = srp[col];
 			buffer[len++] = stp[col];
 			col++;
-			if ((col == TermWin.ncol) || (srp[col] != rend))
+			if ((col == TermWin.bcol) || (srp[col] != rend))
 			    break;
 			if ((stp[col] == dtp[col])
 			    && (srp[col] == drp[col])
@@ -2344,23 +2332,20 @@ scr_refresh(int type)
 	    if (rvid)
 		SWAP_IT(fore, back, i);
 
-#ifdef OFF_FOCUS_FADING
-	    if( rs_fade != NULL && focus_changed)
+	    if( colors_changed)
 	    {
 		gcvalue.background = PixColors[back];
 		gcmask |= GCBackground;
 		gcvalue.foreground = PixColors[fore];
 		gcmask |= GCForeground;
-		focus_changed = 0 ;
-	    }else
-#endif	    
-	    	if (back != Color_bg) {
+		colors_changed = 0 ;
+	    }else if (back != Color_bg) {
 		    gcvalue.background = PixColors[back];
 		    gcmask |= GCBackground;
 		}
 	    if (fore != Color_fg) {
-	        gcvalue.foreground = PixColors[fore];
-	        gcmask |= GCForeground;
+	    	    gcvalue.foreground = PixColors[fore];
+	    	    gcmask |= GCForeground;
 	    }
 #ifndef NO_BOLDUNDERLINE
 	    else if (rend & RS_Bold) {
@@ -2424,7 +2409,7 @@ scr_refresh(int type)
 		DRAW_STRING(draw_string, xpixel, ypixel, buffer, wlen);
 	    } else
 #endif
-#if defined(BACKGROUND_IMAGE)
+#if defined(BACKGROUND_IMAGE) || defined(_MYSTYLE_)
 	    if (TermWin.background.trgPixmapSet && back == Color_bg) {
 		CLEAR_CHARS(xpixel, ypixel - TermWin.font->ascent, len);
 		DRAW_STRING(draw_string, xpixel, ypixel, buffer, wlen);
@@ -2538,6 +2523,31 @@ scr_clear_tint(int bWithTinting)
 #endif
 }
 
+/* ------------------------------------------------------------------------- */
+/* colors has changed so we have to repaint self			     */
+/* PROTO */
+void 
+on_colors_changed(int idx)
+{
+    colors_changed = 1 ;
+    if (idx == Color_bg && !(Options & Opt_transparent))
+    {
+	XSetWindowBackground(Xdisplay, TermWin.vt, PixColors[Color_bg]);
+    }
+
+/* handle Color_BD, scrollbar background, etc. */
+
+    set_colorfgbg();
+    set_cursor_color();
+/* the only reasonable way to enforce a clean update : */
+    scr_clear();
+    scr_touch();
+}
+
+
+
+
+
 
 /* ------------------------------------------------------------------------- */
 /* PROTO */
@@ -2559,7 +2569,7 @@ scr_reverse_selection(void)
 	}
 	end_row += TermWin.nrow;
 	for (; i < row && i < end_row; i++, col = 0)
-	    for (srp = screen.rend[i]; col < TermWin.ncol; col++)
+	    for (srp = screen.rend[i]; col < TermWin.bcol; col++)
 		srp[col] ^= RS_RVid;
 	if (i == row && i < end_row)
 	    for (srp = screen.rend[i]; col < selection.end.col; col++)
@@ -2739,7 +2749,7 @@ selection_make(Time tm)
     if (selection.clicks == 4)
 	return;			/* nothing selected, go away */
 
-    i = (selection.end.row - selection.beg.row + 1) * (TermWin.ncol + 1) + 1;
+    i = (selection.end.row - selection.beg.row + 1) * (TermWin.bcol + 1) + 1;
     str = MALLOC(i * sizeof(char));
     new_selection_text = (unsigned char *) str;
 
@@ -2752,7 +2762,7 @@ selection_make(Time tm)
     for (; row < end_row; row++) {
 	t = &(screen.text[row][col]);
 	if ((end_col = screen.tlen[row]) == -1)
-	    end_col = TermWin.ncol;
+	    end_col = TermWin.bcol;
 	for (; col < end_col; col++)
 	    *str++ = *t++;
 	col = 0;
@@ -2766,7 +2776,7 @@ selection_make(Time tm)
     end_col = screen.tlen[row];
     if (end_col == -1 || selection.end.col <= end_col)
 	end_col = selection.end.col;
-    MIN_IT(end_col, TermWin.ncol);	/* CHANGE */
+    MIN_IT(end_col, TermWin.bcol);	/* CHANGE */
     for (; col < end_col; col++)
 	*str++ = *t++;
     if (end_col != selection.end.col)
@@ -2829,7 +2839,7 @@ selection_start_colrow(int col, int row)
     MAX_IT(selection.mark.row, -TermWin.nscrolled);
     MIN_IT(selection.mark.row, TermWin.nrow - 1);
     MAX_IT(selection.mark.col, 0);
-    MIN_IT(selection.mark.col, TermWin.ncol - 1);
+    MIN_IT(selection.mark.col, TermWin.bcol - 1);
 
     if (selection.op) {		/* clear the old selection */
 	selection.beg.row = selection.end.row = selection.mark.row;
@@ -2872,7 +2882,7 @@ selection_delimit_word(int dirn, row_col_t * mark, row_col_t * ret)
 	dirnadd = -1;
     } else {
 	bound.row = TermWin.saveLines + TermWin.nrow;
-	bound.col = TermWin.ncol - 1;
+	bound.col = TermWin.bcol - 1;
 	dirnadd = 1;
     }
     row = mark->row + TermWin.saveLines;
@@ -2906,7 +2916,7 @@ selection_delimit_word(int dirn, row_col_t * mark, row_col_t * ret)
 	if ((col == bound.col) && (row != bound.row)) {
 	    if (screen.tlen[(row - (dirn == UP))] == -1) {
 		trow = row + dirnadd;
-		tcol = (dirn == UP) ? (TermWin.ncol - 1) : 0;
+		tcol = (dirn == UP) ? (TermWin.bcol - 1) : 0;
 		if (screen.text[trow] == NULL)
 		    break;
 		stp = &(screen.text[trow][tcol]);
@@ -2952,7 +2962,7 @@ selection_extend(int x, int y, int flag)
     MAX_IT(row, 0);
     MIN_IT(row, TermWin.nrow - 1);
     MAX_IT(col, 0);
-    MIN_IT(col, TermWin.ncol);
+    MIN_IT(col, TermWin.bcol);
 #ifndef OLD_SELECTION
 /*
  * If we're selecting characters (single click) then we must check first
@@ -3060,7 +3070,7 @@ selection_extend_colrow(int col, int row, int button3, int buttonpress, int clic
 	} else if (selection.clicks == 3) {
 	    selection.beg.row = selection.end.row = selection.mark.row;
 	    selection.beg.col = 0;
-	    selection.end.col = TermWin.ncol;
+	    selection.end.col = TermWin.bcol;
 	    hate_those_clicks = 1;
 	}
     }
@@ -3085,9 +3095,9 @@ selection_extend_colrow(int col, int row, int button3, int buttonpress, int clic
 	if (ROWCOL_IS_BEFORE(pos, selection.beg)
 	    || (!ROWCOL_IS_AFTER(pos, selection.end)
 		&& (((pos.col - selection.beg.col)
-		     + ((pos.row - selection.beg.row) * TermWin.ncol))
+		     + ((pos.row - selection.beg.row) * TermWin.bcol))
 		    < ((selection.end.col - pos.col)
-		       + ((selection.end.row - pos.row) * TermWin.ncol)))))
+		       + ((selection.end.row - pos.row) * TermWin.bcol)))))
 	    closeto = LEFT;
 	if (closeto == LEFT) {
 	    selection.beg.row = pos.row;
@@ -3122,17 +3132,17 @@ selection_extend_colrow(int col, int row, int button3, int buttonpress, int clic
 	end_col = screen.tlen[selection.beg.row + TermWin.saveLines];
 	if (end_col != -1 && selection.beg.col > end_col) {
 #if 1
-	    selection.beg.col = TermWin.ncol;
+	    selection.beg.col = TermWin.bcol;
 #else
 	    if (selection.beg.row != selection.end.row)
-		selection.beg.col = TermWin.ncol;
+		selection.beg.col = TermWin.bcol;
 	    else
 		selection.beg.col = selection.mark.col;
 #endif
 	}
 	end_col = screen.tlen[selection.end.row + TermWin.saveLines];
 	if (end_col != -1 && selection.end.col > end_col)
-	    selection.end.col = TermWin.ncol;
+	    selection.end.col = TermWin.bcol;
 
 # ifdef MULTICHAR_SET
 	if (selection.beg.col > 0) {
@@ -3142,7 +3152,7 @@ selection_extend_colrow(int col, int row, int button3, int buttonpress, int clic
 		&& ((screen.rend[r][c - 1] & RS_multiMask) == RS_multi1))
 		selection.beg.col--;
 	}
-	if (selection.end.col < TermWin.ncol) {
+	if (selection.end.col < TermWin.bcol) {
 	    r = selection.end.row + TermWin.saveLines;
 	    c = selection.end.col;
 	    if (((screen.rend[r][c - 1] & RS_multiMask) == RS_multi1)
@@ -3159,7 +3169,7 @@ selection_extend_colrow(int col, int row, int button3, int buttonpress, int clic
 	if (ROWCOL_IS_AFTER(selection.mark, selection.beg))
 	    selection.mark.col++;
 	selection.beg.col = 0;
-	selection.end.col = TermWin.ncol;
+	selection.end.col = TermWin.bcol;
     }
     if (button3 && buttonpress) {	/* mark may need to be changed */
 	if (closeto == LEFT) {
@@ -3247,7 +3257,7 @@ void
 pixel_position(int *x, int *y)
 {
     *x = Pixel2Col(*x);
-/* MAX_IT(*x, 0); MIN_IT(*x, TermWin.ncol - 1); */
+/* MAX_IT(*x, 0); MIN_IT(*x, TermWin.bcol - 1); */
     *y = Pixel2Row(*y);
 /* MAX_IT(*y, 0); MIN_IT(*y, TermWin.nrow - 1); */
 }
