@@ -35,9 +35,10 @@
  *----------------------------------------------------------------------*/
 
 #ifndef lint
-static const char rcsid[] = "$Id: xdefaults.c,v 1.12 1998/09/24 18:33:22 mason Exp $";
+static const char rcsid[] = "$Id: xdefaults.c,v 1.5 2000/05/30 22:17:56 sashav Exp $";
 #endif
 
+#include "libafterstep.h"
 #include "rxvt.h"		/* NECESSARY */
 
 /* #define DEBUG_RESOURCES */
@@ -139,6 +140,10 @@ static const struct {
     STRG(rs_geometry, NULL, "g", NULL, NULL),	/* short form */
     BOOL(rs_reverseVideo, "reverseVideo", "rv", Opt_reverseVideo,
          "reverse video"),
+#ifdef _MYSTYLE_	 
+    STRG(rs_mystyle, "MyStyle", "mst", "name",
+         "MyStyle"),
+#endif
     STRG(rs_color[Color_bg], "background", "bg", "color",
          "background color"),
     STRG(rs_color[Color_fg], "foreground", "fg", "color",
@@ -291,6 +296,8 @@ static const struct {
 #endif				/* NO_CURSORCOLOR */
     STRG(rs_color[Color_pointer], "pointerColor", "pr", "color",
          "pointer color"),
+    STRG(rs_borderWidth, "borderWidth", "bw", "number",
+         "width of border"),
     STRG(rs_color[Color_border], "borderColor", "bd", "color",
          "border color"),
     BOOL(rs_loginShell, "loginShell", "ls", Opt_loginShell, "login shell"),
@@ -303,6 +310,8 @@ static const struct {
          Opt_scrollTtyOutputInh, "scroll-on-tty-output inhibit"),
     BOOL(rs_scrollKeypress, "scrollTtyKeypress", "sk",
          Opt_scrollKeypress, "scroll-on-keypress"),
+    STRG(rs_minBufferWidth, "minBufferWidth", "mbw", "number",
+         "minimum number of columns stored in buffer"),
     STRG(rs_saveLines, "saveLines", "sl", "number",
          "number of scrolled lines to save"),
     BOOL(rs_utmpInhibit, "utmpInhibit", "ut", Opt_utmpInhibit,
@@ -354,6 +363,8 @@ list_options()
 #define INDENT 30
     fprintf(stderr, "(");
 #ifdef BACKGROUND_IMAGE
+    fprintf(stderr, "background image,");
+#endif
 #ifdef XPM
     fprintf(stderr, "XPM,");
 #endif
@@ -364,7 +375,11 @@ list_options()
 #ifdef PNG
     fprintf(stderr, "PNG,");
 #endif
-    fprintf(stderr, "using AfterStep imaging,");
+    fprintf(stderr, "AfterStep integration,");
+#ifdef _MYSTYLE_
+    fprintf(stderr, "AfterStep MyStyles,");
+#else
+    fprintf(stderr, "no AfterStep MyStyles,");
 #endif
 #endif
 #ifdef UTMP_SUPPORT
